import { ValidateRelationHelper } from 'src/core/helpers/validation/validate-relation.helper';
import { BaseManager } from '../base.manager';
import { OPERATION, STATUS } from 'src/core/strings/constants/base.constants';
import * as _ from 'lodash';
import { RecordLog } from 'src/modules/configuration/log/domain/entities/log.event';

export abstract class BaseUpdateStatusManager<Entity> extends BaseManager {
  protected dataId: string;
  protected result: Entity;
  protected oldData: Entity;
  protected dataStatus: STATUS;
  protected duplicateColumn: string[];
  abstract get entityTarget(): any;

  setData(id: string, status: STATUS): void {
    this.dataId = id;
    this.dataStatus = status;
  }

  async prepareData(): Promise<void> {
    this.data = await this.dataService.getOneByOptions({
      where: {
        id: this.dataId,
      }
    })
    this.oldData = _.cloneDeep(this.data);

    Object.assign(this.data, {
      editor_id: this.user.id,
      editor_name: this.user.name,
      updated_at: new Date().getTime(),
      status: this.dataStatus,
    });
  }

  async process(): Promise<void> {
    await new ValidateRelationHelper(
      this.dataId,
      this.dataService,
      this.validateRelations,
      this.tableName,
    ).execute();

    this.result = await this.dataService.update(
      this.queryRunner,
      this.entityTarget,
      { id: this.dataId },
      this.data,
    );

    this.publishEvents();
  }

  abstract getResult(): string;

  async publishEvents() {
    this.eventBus.publish(
      new RecordLog({
        id: this.result['id'],
        old: this.oldData,
        data: this.result,
        user: this.user,
        description: `${ this.user.name } update status data ${ this.tableName } to ${ this.dataStatus }`,
        module: this.tableName,
        op: OPERATION.UPDATE,
      }),
    );

    if (!this.eventTopics.length) return;
    for (const topic of this.eventTopics) {

      let data;
      if (!topic.data) {
        data = await this.dataService.getOneByOptions({
          where: {
            id: this.dataId
          },
          relations: topic.relations,
        })
      }

      this.eventBus.publishAll([
        new topic.topic({
          id: data?.['id'] ?? topic?.data?.['id'],
          old: this.oldData,
          data: data ?? topic.data,
          user: this.user,
          description: '',
          module: this.tableName,
          op: OPERATION.UPDATE,
        }),
      ]);
    }
  }
}
