import { Injectable } from '@nestjs/common';
import {
  EventTopics,
  columnUniques,
  validateRelations,
} from 'src/core/strings/constants/interface.constants';
import { TransactionEntity } from '../../entities/transaction.entity';
import { TransactionModel } from '../../../data/models/transaction.model';
import { BaseCreateManager } from 'src/core/modules/domain/usecase/managers/base-create.manager';
import { TransactionCreatedEvent } from '../../entities/event/transaction-created.event';
import { TransactionType } from '../../../constants';

@Injectable()
export class CreateTransactionManager extends BaseCreateManager<TransactionEntity> {
  async beforeProcess(): Promise<void> {
    Object.assign(this.data, {
      type: TransactionType.ADMIN,
      customer_category_id: this.data.customer_category?.id ?? null,
      customer_category_name: this.data.customer_category?.name ?? null,
      season_period_id: this.data.season_period?.id ?? null,
      season_period_name: this.data.season_period?.holiday_name ?? null,
      season_period_type_id: this.data.season_period?.season_type?.id ?? null,
      season_period_type_name:
        this.data.season_period?.season_type?.name ?? null,
    });

    this.data.items?.map((item) => {
      Object.assign(item, {
        item_id: item.item.id,
        item_name: item.item.name,
        item_type: item.item.item_type,
        item_price: item.item.base_price,
        item_tenant_id: item.item.tenant?.id ?? null,
        item_tenant_name: item.item.tenant?.id ?? null,
        item_tenant_percentage: item.item.tenant?.share_margin ?? null,
      });
    });

    console.log(this.data, 'data');
    throw new Error('das');

    return;
  }

  async afterProcess(): Promise<void> {
    return;
  }

  get validateRelations(): validateRelations[] {
    return [];
  }

  get uniqueColumns(): columnUniques[] {
    return [];
  }

  get eventTopics(): EventTopics[] {
    return [
      {
        topic: TransactionCreatedEvent,
        data: this.data,
      },
    ];
  }

  get entityTarget(): any {
    return TransactionModel;
  }
}
