import { Module } from '@nestjs/common';
import { ConfigModule } from '@nestjs/config';
import { TypeOrmModule } from '@nestjs/typeorm';
import { CONNECTION_NAME } from 'src/core/strings/constants/base.constants';
import { TransactionDataService } from './data/services/transaction-data.service';
import { TransactionReadService } from './data/services/transaction-read.service';
import { TransactionReadController } from './infrastructure/transaction-read.controller';
import { TransactionReadOrchestrator } from './domain/usecases/transaction-read.orchestrator';
import { TransactionDataController } from './infrastructure/transaction-data.controller';
import { TransactionDataOrchestrator } from './domain/usecases/transaction-data.orchestrator';
import { CreateTransactionManager } from './domain/usecases/managers/create-transaction.manager';
import { CqrsModule } from '@nestjs/cqrs';
import { IndexTransactionManager } from './domain/usecases/managers/index-transaction.manager';
import { DeleteTransactionManager } from './domain/usecases/managers/delete-transaction.manager';
import { UpdateTransactionManager } from './domain/usecases/managers/update-transaction.manager';
import { ActiveTransactionManager } from './domain/usecases/managers/active-transaction.manager';
import { ConfirmTransactionManager } from './domain/usecases/managers/confirm-transaction.manager';
import { InactiveTransactionManager } from './domain/usecases/managers/inactive-transaction.manager';
import { DetailTransactionManager } from './domain/usecases/managers/detail-transaction.manager';
import { BatchDeleteTransactionManager } from './domain/usecases/managers/batch-delete-transaction.manager';
import { BatchActiveTransactionManager } from './domain/usecases/managers/batch-active-transaction.manager';
import { BatchConfirmTransactionManager } from './domain/usecases/managers/batch-confirm-transaction.manager';
import { BatchInactiveTransactionManager } from './domain/usecases/managers/batch-inactive-transaction.manager';
import { TransactionModels } from './constants';

@Module({
  imports: [
    ConfigModule.forRoot(),
    TypeOrmModule.forFeature(TransactionModels, CONNECTION_NAME.DEFAULT),
    CqrsModule,
  ],
  controllers: [TransactionDataController, TransactionReadController],
  providers: [
    IndexTransactionManager,
    DetailTransactionManager,
    CreateTransactionManager,
    DeleteTransactionManager,
    UpdateTransactionManager,
    ActiveTransactionManager,
    ConfirmTransactionManager,
    InactiveTransactionManager,
    BatchDeleteTransactionManager,
    BatchActiveTransactionManager,
    BatchConfirmTransactionManager,
    BatchInactiveTransactionManager,

    TransactionDataService,
    TransactionReadService,

    TransactionDataOrchestrator,
    TransactionReadOrchestrator,
  ],
})
export class TransactionModule {}
